%function [value] = fnext_prmulti_pen_g_ni_m(headerpoint, valuespoint, mu_TETA,
%var_TETA, M, w)
%returns Probability Exceeding Penalty for Gain multiple: 
%1-Pr{G_1(x,teta_1) <= w, G_2(x,teta_2) <= w, ..., G_M(x,teta_M) <= w }
%where 
%G_m(x,teta_m) = -teta_m_0 + x_1*teta_m_1 + ... +x_I*teta_m_I, m=1,...,M
%are gain functions. For fixed m 
%teta__m_0, teta_m_1, ..., teta_m_I - are normally distributed independent
%random coefficients with vector of averages  mu_TETA_m, and 
%vector of variances var_TETA_m,    m = 1, 2,...,M
% Arguments:
%headerpoint = a string with the tabseparated list of variables names ;
%valuespoint = real vector (row vector) with components
%ordered  as the sequence of variables passed as the first argument
%(headerpoint);
%M = number of gain functions
% mu_TETA = matrix with M rows and I+1 columns. The row # m contains
%vector of averages mu_TETA_m. The first component of  mu_TETA_m is
%the average of coefficient teta_0, other components are ordered 
%in accordance with the headerpoint
%var_TETA = matrix with M rows and I+1 columns. The row # m contains
%vector var_TETA_m of  variances of coefficients of the loss function 
%L_m(x,teta_m)
%w = threshold used for calculation of Probability Exceeding Penalty for 
%Gain multiple: 
%1-Pr{G_1(x,teta_1) <= w, G_2(x,teta_2) <= w, ..., G_M(x,teta_M) <= w }



function [value] = fnext_prmulti_pen_g_ni_m(headerpoint, valuespoint, mu_TETA, var_TETA,M, w)
    global CDF_table_0_1;
    global PDF_table_0_1;

    Z = [-1 +valuespoint];
    product = 1;
    for m=1:1:M
        mu = Z*(mu_TETA(m,:))';
        sigma =sqrt(var_TETA(m,:)*(Z.^2)');
        product = product*NormalCDF(w,mu, sigma,CDF_table_0_1);
    end

    value = 1 - product;
%    fprintf('ext func = %.12e', value);
return;

%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2014.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions       |
%and derivative works.                                                  |
%                                                                       |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|
